@@ -17,7 +17,43 @@ let server_comments request t =
 
 let qs1 = {| quoted string |}   (* (issue #24) *)
 let qs2 = {eof| other quoted string   |noteof}  |eof}
-    
+
+(* ocp-indent does it as follows:
+let test1 = with_connection (fun conn ->
+    do_something conn x;
+    ...
+  )
+    toto
+ *)
+let test1 = with_connection (fun conn ->
+                do_something conn x;
+                ...
+              )
+              toto
+
+let x = match y with            (* Issue #71 *)
+  | A | B ->
+     do_something ()
+
+let x = match y, z with
+  | A, (B | C)
+  | X, Y -> do_something()      (* Issue #78 *)
+
+let x =
+  begin match y with
+  | A -> 1                       (* Issue #73 *)
+  end
+
+(* The two "let"s below are indented under the assumption that
+   tuareg-indent-align-with-first-arg is nil!  *)
+let x = List.map (fun x -> 5)
+          my list
+
+let x =
+  logf `Info "User %s has %i new messages" ba
+    (Uid.to_string uid)
+    (List.length new_messages);
+
 let x =
   let open M in
   let x = 5 in
@@ -94,9 +130,9 @@ type t = [ `Foo of int
          | `Bar of string ]
 
 type t =
-| A
-| B
-| C
+  | A
+  | B  (* issue #76 *)
+  | C
 with sexp
 
 type t = | A
@@ -104,51 +140,56 @@ type t = | A
          | C
 
 type t = [
-| `A
-| `B
-| `C
-]
-
-type t = [                              (* Comment.  *)
-| `A
-| `B
-| `C
-]
-
-type t = a
-and x = b
-
-module M = struct
-  type t =
-  | A
-  | B
-  | C
-  with sexp
-
-  type s = [
   | `A
   | `B
   | `C
   ]
 
+type t = [                              (* Comment.  *)
+  | `A
+  | `B
+  | `C
+  ]
+
+type t = a
+and typey = 4
+and x = b
+
+module M = struct
+  type t =
+    | A
+    | B
+    | C
+  with sexp
+
+  type s = [
+    | `A
+    | `B
+    | `C
+    ]
+
   type u =
-  | D
-  | E
+    | D
+    | E
   with sexp
 end
 
 module N =
 struct
   type u =
-  | D
-  | E
+    | D
+    | E
   with sexp
 end
 
 type m =
-| T
+  | T
 with sexp
 
+let f = function
+  | A -> 1
+  | B | C -> 2
+
 ;; (* http://caml.inria.fr/mantis/view.php?id=4334 *)
 type foo =
   a
@@ -343,6 +384,9 @@ let rec count_append l1 l2 count =
        (if count > 1000
         then slow_append tl l2
         else count_append tl l2 (count + 1))
+  (* New in OCaml-4.02.  *)
+  | exception Not_Found ->
+     l2
 
 
 let x =
@@ -372,8 +416,8 @@ let foo =
   )
 
 let quux list = List.map list ~f:(fun item ->
-  print_item item
-)
+                    print_item item
+                  )
 
 let foo x = function
   | Some _ -> true
@@ -437,11 +481,11 @@ let d x = function
      I chose with "match" because it looks otherwise odd and is more
      consistent with the "try" alignments above.  *)
   | A -> (match x with
-         | X ->
-            false
-         | Y -> true
-         |  Z ->
-             false)
+          | X ->
+             false
+          | Y -> true
+          |  Z ->
+              false)
   | B -> false
 
 let a f = function
@@ -451,17 +495,17 @@ let a f = function
     2
   |      C ->
     (function
-    |  X  ->
-      a
-    | Y ->
-      b) 12
+     |  X  ->
+         a
+     | Y ->
+        b) 12
   | D ->
-    (match z with
-     | 4 -> 3
-     |  5 -> 7)
+     (match z with
+      | 4 -> 3
+      |  5 -> 7)
 
 let x = foo ~f:(fun _ -> 0              (* Comment.  *)
-)
+          )
 
 let f x =
   (let y = x in
@@ -547,11 +591,11 @@ let () =
 let () =
   try f a
   with A () ->
-    ()
-  | B () ->
-    ()
-  |     C () ->
-    ()
+       ()
+     | B () ->
+        ()
+     |     C () ->
+            ()
 
 let f errors input =
   let ( @@ ) string bool = if not bool then errors := string :: !errors in
